from pathlib import Path

from hatchling.plugin import hookimpl
from hatchling.version.source.plugin.interface import VersionSourceInterface
from versioning import pep440, semver2
from versioning.project import NoVersion


class SimpleGitVersioningVersionSource(VersionSourceInterface):  # pragma: no cover
    PLUGIN_NAME = "simple-git-versioning"

    def get_version_data(self) -> dict:
        flavor = self.config["flavor"]

        if not isinstance(flavor, str):
            raise TypeError(f"unexpected versioning flavor: '{flavor}', expected 'pep440' or 'semver2'")

        flavor = flavor.casefold()
        if flavor == "pep440":
            Project = pep440.Project
            options = dict(dev=0)
        elif flavor == "semver2":
            Project = semver2.Project
            options = dict()
        else:
            raise ValueError(
                f"unexpected value for `tool.simple-git-versioning.setuptools`: '{flavor}', expected 'pep440' or 'semver2'"
            )

        with Project(path=Path(self.root)) as proj:
            try:
                return {"version": str(proj.version())}
            except NoVersion:
                return {"version": str(proj.release(**options))}


@hookimpl
def hatch_register_version_source():  # pragma: no cover
    return SimpleGitVersioningVersionSource
